class _EnglishVocabMeta:

  @classmethod
  def get_pos(cls, dict_pos):
    _pos = 'other'
    for pos, des in cls.pos_info.items():
      if dict_pos in des[2]:
        _pos = pos
        break
    return _pos 
  
  @classmethod
  def get_pos_name(cls, pos):
    return cls.pos_info[pos][1]

  @classmethod
  def get_pos_cn_name(cls, pos):
    return cls.pos_info[pos][0]

  pos_info = {
    'noun': ('名词', 'noun', ['n.']),
    'adj': ('形容词', 'adjective', ['adj.']),
    'verb': ('动词', 'verb', ['v.', 'vt.vi.', 'v.vi.', 'v.vt.', 'v.aux.', 'vi.vt.']),
    'adv': ('副词', 'adverb', ['adv.']),
    'pron': ('代词', 'pronoun', ['pron.']),
    'prep': ('前置词', '', ['prep.']),
    'other': ('其他', 'others', [])
  }

  format_info = {
    'original': "原型",
    'present_participle': "现在分词",
    'past_participle': "过去分词",
    'past_tense': "过去式",
    '3': "三单",
    'comparative': "比较级",
    'superlative': "最高级",
    'singular': "单数",
    'plural': "复数"
  }





from linguappt.vocab_ppt import VocabPPT
from linguappt.es.vocab_meta import SpanishVocabMeta
import os
import json

class SpanishVocabPPT(VocabPPT):
  """Create Vocabulary PPT for Spanish study

  Attributes:
    content (list of dict): read from csv file
    word_distrubtion (dict): key is PoS, e.g, ``noun``, ``verb``, ``daj``, value is list of vocabularies
  """

  _template_dir = os.path.dirname(__file__)
  _templates = {
    "classic": os.path.join(_template_dir, 'templates/vocab_spanish_classic.pptx'),
  }
  lang = 'es'

  content_keys = ['word', 'meaning', 'dict_pos', 'from', 'extension', 'variations']

  _metainfo = SpanishVocabMeta

  ALLOWED_POSES = ['noun', 'adj', 'verb']

  def __init__(self, sourcefile, title="", genre="classic"):
    super().__init__(sourcefile, title, genre)

  def _create_noun_with_extension_A(self, v):
    layout = self._prs.slide_layouts.get_by_name("Noun with extension A")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders

    pos = v["dict_pos"]

    pos_holder = holders[10]
    pos_holder.text_frame.text = pos 
    noun, meaning = holders[11], holders[12]
    noun.text_frame.text = v["word"]
    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]
    meaning.text_frame.text = "\n".join(ms) 


    s_def = holders[13]
    s_undef = holders[15]
    pl_def = holders[17]
    pl_undef = holders[19]

    arts = self.__class__._metainfo.article_variation[pos]

    s_def.text_frame.text = arts[0] 
    s_undef.text_frame.text = arts[1]
    pl_def.text_frame.text = arts[2]
    pl_undef.text_frame.text = arts[3]

    extension = json.loads(v["extension"])
    if "pl." in pos:
      s, pl = extension[arts[4]], v["word"]
    else:
      s, pl = v["word"], extension[arts[4]]     

    holders[14].text_frame.text = s
    holders[16].text_frame.text = s
    holders[18].text_frame.text = pl 
    holders[20].text_frame.text = pl 

    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]

  def _create_noun(self, v):
    if v["extension"] != "":
      self._create_noun_with_extension_A(v)
    else:
      self._create_default_word(v)

  def _create_adj_with_extension_A(self, v):
    layout = self._prs.slide_layouts.get_by_name("Adj with extension A")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders

    pos_holder = holders[10]
    adj, meaning = holders[11], holders[12]
    adj.text_frame.text = v["word"]
    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]

    meaning.text_frame.text = "\n".join(ms)

    s_m, s_f, pl_m, pl_f = holders[13], holders[14], holders[15], holders[16]

    extension = json.loads(v["extension"])

    s_m.text_frame.text = extension["m"]
    s_f.text_frame.text = extension["f"]
    pl_m.text_frame.text = extension["mpl"]
    pl_f.text_frame.text = extension["fpl"]

    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]

  def _create_adj(self, v):
    if v["extension"] != "":
      self._create_adj_with_extension_A(v)
    else:
      self._create_default_word(v)

  def _create_verb_with_extension_A(self, v):
    layout = self._prs.slide_layouts.get_by_name("Verb with extension A")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders
    
    variations = json.loads(v["variations"])

    pos_holder = holders[10]
    pos_holder.text_frame.text = v["dict_pos"]

    origin, word, meaning = holders[11], holders[12], holders[13]
    origin.text_frame.text = variations["origin"]
    word.text_frame.text = v["word"]

    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]

    meaning.text_frame.text = "\n".join(ms)

    sign = variations["formats"][0]
    tense = sign["tense"]
    person = sign["person"]

    extension = json.loads(v["extension"])[tense]

    holders[14].text_frame.text = extension["yo"] if extension["yo"] != "" else " "
    holders[15].text_frame.text = extension["tú"]
    holders[16].text_frame.text = extension["él/ella/Usted"]
    holders[17].text_frame.text = extension["nosotros"]
    holders[18].text_frame.text = extension["vosotros"]
    holders[19].text_frame.text = extension["ellos/ellas/Ustedes"]

    holders[20].text_frame.text = self.__class__._metainfo.tense_info[tense]
    holders[21].text_frame.text = " ".join(["人称", person, "的变位"])

    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]


  def _create_verb_with_extension_B(self, v):
    layout = self._prs.slide_layouts.get_by_name("Verb with extension B")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders
    
    variations = json.loads(v["variations"])

    pos_holder = holders[10]
    pos_holder.text_frame.text = v["dict_pos"]

    origin, word, meaning = holders[11], holders[12], holders[13]
    origin.text_frame.text = variations["origin"]
    word.text_frame.text = v["word"]

    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]

    meaning.text_frame.text = "\n".join(ms)
 
    formats = variations["formats"]
    holders[14].text_frame.text = "\n".join([self.__class__._metainfo.tense_info[f["tense"]] if "tense" in f.keys() else self.__class__._metainfo.tense_info[f["format"]] for f in formats])
    holders[15].text_frame.text = "\n".join([" ".join([f["person"], "的变位"]) if "person" in f.keys() else "" for f in formats])

    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]

  def _create_verb_with_extension_C(self, v):
    layout = self._prs.slide_layouts.get_by_name("Verb with extension C")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders
    
    variations = json.loads(v["variations"])

    pos_holder = holders[10]
    pos_holder.text_frame.text = v["dict_pos"]
    origin, word, meaning = holders[11], holders[12], holders[13]
    origin.text_frame.text = variations["origin"]
    word.text_frame.text = v["word"]

    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]

    meaning.text_frame.text = "\n".join(ms)
 
    sign = variations["formats"][0]

    holders[14].text_frame.text = self.__class__._metainfo.tense_info[sign["format"]]

    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]

  def _create_verb(self, v):
    if v["variations"] == "":
      self._create_default_word(v)
    else:
      variations = json.loads(v["variations"])
      if "formats" in variations.keys():
        formats = variations["formats"]
        if len(formats) > 1:
          self._create_verb_with_extension_B(v)
        elif "tense" in formats[0].keys():
          self._create_verb_with_extension_A(v)
        elif "format" in formats[0].keys():
          self._create_verb_with_extension_C(v)
        else:
          self._create_default_word(v)
      else:
        self._create_default_word(v)




class SpanishVocabMeta:
  @classmethod
  def get_pos(cls, dict_pos):
    _pos = 'other'
    for pos, des in cls.pos_info.items():
      if dict_pos in des[2]:
        _pos = pos
        break
    return _pos 
  
  @classmethod
  def get_pos_name(cls, pos):
    return cls.pos_info[pos][1]

  @classmethod
  def get_pos_cn_name(cls, pos):
    return cls.pos_info[pos][0]

  pos_info = {
    'noun': ('名词', 'el nombre', ['noun.', 'f.', 'm.', 'f.m.', 'propn,', 'f.pl.', 'm.pl.']),
    'adj': ('形容词', 'el adjectivo', ['adj.']),
    'verb': ('动词', 'el verbo', ['verb.', 'vr.', 'vi.', 'vt.', 'aux.']),
    'adv': ('副词', 'el adverbio', ['adv.']),
    'pron': ('代词', 'los pronombres', ['pron.']),
    'prep': ('前置词', '', ['prep.', 'adp.']),
    'other': ('其他', 'los otros', [])
  }

  tense_info = {
    'imperativo_afirmativo': "命令式-肯定",
    'imperativo_negativo': "命令式-否定",
    'indicativo-pretérito': "陈述式-过去时",
    'indicativo-presente': "陈述式-现在时",
    'subjuntivo-presente': "虚拟式-现在时",
    'indicativo-futuro': "陈述式-将来时",
    'subjuntivo-futuro': "虚拟式-将来时",
    'indicativo-imperfecto': "陈述式-未完成时",
    'subjuntivo-imperfecto': "虚拟式-未完成时",
    'indicativo-condicional': "条件式",
    'participio': "过去分词",
    'gerundio': "现在分词"
  }

  article_variation = {
    "m.": ["un", "el", "unos", "los", "mpl"],
    "m.pl.": ["un", "el", "unos", "los", "m"],
    "f.": ["una", "la", "unas", "las", "fpl"],
    "f.pl.": ["una", "la", "unas", "las", "f"],
  }



"""``linguappt`` Package

This module demostrates the usage of package `linguappt`.

.. topic:: Installation

  .. code:: shell

    $ pip3 install linguappt

.. topic:: Use as executable

  .. code:: shell

    $ lingua_vocabppt --sourcecsv [vocab csv file] --lang [language] --title [title shown in ppt] --destpptx [pptx file]

    $ lingua_phraseppt --sourcecsv [phrase csv file] --lang [language] --title [title shown in ppt] --destpptx [pptx file]

    $ lingua_pptx2pdf --sourcepptx [pptx file] --destdir [dest directory storing pdf and images]

.. topic:: Convert csv file containing vocabulary informatin into pptx file 

  .. code:: python

    from linguappt import SpanishVocabPPT, EnglishVocabPPT

    def vocabppt(sourcecsv, title, lang, destpptx):
      _PPTS = {
        "en": EnglishVocabPPT,
        "es": SpanishVocabPPT
      }

      _PPT = _PPTS[lang]

      vp = _PPT(sourcecsv, title)
      vp.convert_to_ppt(destpptx)

.. topic:: Convert csv file containing vocabulary informatin into pptx file 

  .. code:: python

    from linguappt import EnglishPhrasePPT, SpanishPhrasePPT

    def phraseppt(sourcecsv, title, lang, destpptx):
      _PPTS = {
        "en": EnglishPhrasePPT,
        "es": SpanishPhrasePPT
      }

      _PPT = _PPTS[lang]

      vp = _PPT(sourcecsv, title)
      vp.convert_to_ppt(destpptx)

"""

__version__ = '0.1.11'


from .es.vocab_summary import SpanishVocabPPT
from .en.vocab_summary import EnglishVocabPPT
from .en.phrase_summary import EnglishPhrasePPT
from .es.phrase_summary import SpanishPhrasePPT



from linguappt.vocab_ppt import VocabPPT
from linguappt.en._vocab_meta import _EnglishVocabMeta
import os
import json

class EnglishVocabPPT(VocabPPT):
  """Create Vocabulary PPT for English study

  Attributes:
    content (list of dict): read from csv file
    word_distrubtion (dict): key is PoS, e.g, ``noun``, ``verb``, ``daj``, value is list of vocabularies
  """

  _template_dir = os.path.dirname(__file__)
  _templates = {
    "classic": os.path.join(_template_dir, 'templates/vocab_english_classic.pptx'),
  }
  lang = 'en'

  content_keys = ['word', 'meaning', 'dict_pos', 'from', 'extension', 'variations', 'examples']

  _metainfo = _EnglishVocabMeta

  ALLOWED_POSES = ['noun', 'adj', 'verb']

  def __init__(self, sourcefile, title="", genre="classic"):
    super().__init__(sourcefile, title, genre)

  def _create_noun_with_extension_B(self, v):
    layout = self._prs.slide_layouts.get_by_name("Noun with extension B")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders

    format_info = self.__class__._metainfo.format_info

    pos, noun, meaning = holders[10], holders[11], holders[12]
    description = format_info["singular"]
    if v["variations"] != "":
      variations = json.loads(v["variations"])
      description = " ".join([format_info[e] for e in variations["formats"]])

    pos.text_frame.text = " ".join([v["dict_pos"], description])
    noun.text_frame.text = v["word"]
    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]
    meaning.text_frame.text = "\n".join(ms) 

    extension = json.loads(v["extension"])
    single, plural = holders[13], holders[14]
    single.text_frame.text = extension["singular"]
    plural.text_frame.text = extension["plural"]

    examples = json.loads(v["examples"])
    if len(examples) == 2:
      original, translated = holders[15], holders[16]
      ex = examples[0]
      original.text_frame.text = ex["original"]
      translated.text_frame.text = ex["translated"]

      original, translated = holders[17], holders[18]
      ex = examples[1]
      original.text_frame.text = ex["original"]
      translated.text_frame.text = ex["translated"]

    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]

  def _create_noun_with_extension_A(self, v):
    layout = self._prs.slide_layouts.get_by_name("Noun with extension A")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders

    format_info = self.__class__._metainfo.format_info

    pos, noun, meaning = holders[10], holders[11], holders[12]
    description = format_info["singular"]
    if v["variations"] != "":
      variations = json.loads(v["variations"])
      description = " ".join([format_info[e] for e in variations["formats"]])
    pos.text_frame.text = " ".join([v["dict_pos"], description])

    noun.text_frame.text = v["word"]
    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]
    meaning.text_frame.text = "\n".join(ms) 

    extension = json.loads(v["extension"])
    single, plural = holders[13], holders[14]
    single.text_frame.text = extension["singular"]
    plural.text_frame.text = extension["plural"]

    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]

  def _create_noun(self, v):
    extension, variations, examples = v["extension"], v["variations"], v["examples"]
    if extension != "" and variations != "" and examples != "[]":
      self._create_noun_with_extension_B(v)
    elif extension != "" and examples == "[]":
      self._create_noun_with_extension_A(v)
    elif extension == "" and examples != "[]":
      self._create_default_word_with_examples(v)
    else:
      self._create_default_word(v)

  def _create_adj_with_extension_B(self, v):
    layout = self._prs.slide_layouts.get_by_name("Adj with extension B")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders

    format_info = self.__class__._metainfo.format_info

    pos, adj, meaning = holders[10], holders[11], holders[12]
    description = ""
    if v["variations"] != "":
      variations = json.loads(v["variations"])
      description = " ".join([format_info[e] for e in variations["formats"]])
    pos.text_frame.text = " ".join([v["dict_pos"], description])
    adj.text_frame.text = v["word"]
    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]

    meaning.text_frame.text = "\n".join(ms)

    much, more, most = holders[12], holders[13], holders[14]
    extension = json.loads(v["extension"])

    much.text_frame.text = extension['original'] 
    more.text_frame.text = extension['comparative']
    most.text_frame.text = extension['superlative']

    examples = json.loads(v["examples"])
    if len(examples) >= 1:
      original, translated = holders[15], holders[16]
      ex = examples[0]
      original.text_frame.text = ex["original"]
      translated.text_frame.text = ex["translated"]
    if len(examples) >= 2:
      original, translated = holders[17], holders[18]
      ex = examples[1]
      original.text_frame.text = ex["original"]
      translated.text_frame.text = ex["translated"]
 
    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]


  def _create_adj_with_extension_A(self, v):
    layout = self._prs.slide_layouts.get_by_name("Adj with extension A")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders

    format_info = self.__class__._metainfo.format_info

    pos, adj, meaning = holders[10], holders[11], holders[12]
    description = ""
    if v["variations"] != "":
      variations = json.loads(v["variations"])
      description = " ".join([format_info[e] for e in variations["formats"]])
    pos.text_frame.text = " ".join([v["dict_pos"], description])
    adj.text_frame.text = v["word"]
    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]

    meaning.text_frame.text = "\n".join(ms)

    much, more, most = holders[13], holders[14], holders[15]
    extension = json.loads(v["extension"])

    much.text_frame.text = extension['original'] 
    more.text_frame.text = extension['comparative']
    most.text_frame.text = extension['superlative']

    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]


  def _create_adj(self, v):
    extension, variations, examples = v["extension"], v["variations"], v["examples"]
    if extension != "" and variations != "" and examples != "[]":
      self._create_adj_with_extension_B(v)
    elif extension != "" and examples == "[]":
      self._create_adj_with_extension_A(v)
    elif extension == "" and examples != "[]":
      self._create_default_word_with_examples(v)
    else:
      self._create_default_word(v)

  def _create_verb_with_extension_B(self, v):
    layout = self._prs.slide_layouts.get_by_name("Verb with extension B")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders

    format_info = self.__class__._metainfo.format_info

    pos, verb, meaning = holders[10], holders[11], holders[12]
    description = ""
    if v["variations"] != "":
      variations = json.loads(v["variations"])
      description = " ".join([format_info[e] for e in variations["formats"]])
    pos.text_frame.text = " ".join([v["dict_pos"], description])
    verb.text_frame.text = v["word"]
    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]

    meaning.text_frame.text = "\n".join(ms)

    original, present_participle, past_tense, past_participle, third  = holders[13], holders[14], holders[15], holders[16], holders[17]
    extension = json.loads(v["extension"])

    original.text_frame.text = extension['original'] if 'original' in extension else " "
    past_tense.text_frame.text = extension['past_tense'] if 'past_tense' in extension else " "
    third.text_frame.text = extension['3'] if '3' in extension else " "
    present_participle.text_frame.text = extension['present_participle']  if 'present_participle' in extension else " "
    past_participle.text_frame.text = extension['past_participle'] if 'past_participle' in extension else " "

    examples = json.loads(v["examples"])
    if len(examples) >= 1:
      original, translated = holders[18], holders[19]
      ex = examples[0]
      original.text_frame.text = ex["original"]
      translated.text_frame.text = ex["translated"]
    if len(examples) >= 2:
      original, translated = holders[20], holders[21]
      ex = examples[1]
      original.text_frame.text = ex["original"]
      translated.text_frame.text = ex["translated"]
 
    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]

  def _create_verb_with_extension_A(self, v):
    layout = self._prs.slide_layouts.get_by_name("Verb with extension A")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders

    format_info = self.__class__._metainfo.format_info

    pos, verb, meaning = holders[10], holders[11], holders[12]
    description = ""
    if v["variations"]:
      variations = json.loads(v["variations"])
      description = " ".join([format_info[e] for e in variations["formats"]])
    pos.text_frame.text = " ".join([v["dict_pos"], description])
    verb.text_frame.text = v["word"]
    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]

    meaning.text_frame.text = "\n".join(ms)

    original, past_tense, present_participle, past_participle, third = holders[13], holders[14], holders[15], holders[16], holders[17]
    extension = json.loads(v["extension"])

    original.text_frame.text = extension['original'] if 'original' in extension else " "
    past_tense.text_frame.text = extension['past_tense'] if 'past_tense' in extension else " "
    third.text_frame.text = extension['3'] if '3' in extension else " "
    present_participle.text_frame.text = extension['present_participle']  if 'present_participle' in extension else " "
    past_participle.text_frame.text = extension['past_participle'] if 'past_participle' in extension else " "

    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]

  def _create_verb(self, v):
    extension, variations, examples = v["extension"], v["variations"], v["examples"]
    if extension != "" and variations != "" and examples != "[]":
      self._create_verb_with_extension_B(v)
    elif extension != "" and examples == "[]":
      self._create_verb_with_extension_A(v)
    elif extension == "" and examples != "[]":
      self._create_default_word_with_examples(v)
    else:
      self._create_default_word(v)





from pptx import Presentation
from pptx.util import Inches, Pt
from pptx.dml.color import RGBColor

import os

import click
@click.command()
@click.option("--pptx", prompt="pptx file", help="Specify the pptx file to be validated")
def validate(pptx):
  prs = Presentation(pptx)

  templates = prs.slide_layouts
  for index, t in enumerate(templates):
    slide = prs.slides.add_slide(t)
    holders = slide.shapes.placeholders

    print('Template {}-{}'.format(index+1, t.name))
    for e in holders:
      print('  %d %s' % (e.placeholder_format.idx, e.name))
   
